/**
 * \file MP3ButtonPlayer2.ino
 * \This is a modified version an MP3 shield sketch originally written by Michael P. Flaga
 *
 * This sketch utilizes the analog and digital input pins to select 8 different MP3 tracks 
 * Tracks must be labeled track001.mp3 through track008.mp3 

 * \note Use this example uses the bounce2 library to provide debouncing fuctions. Advocated by Arduino's website at http://playground.arduino.cc/code/bounce
 */

// libraries
#include <SPI.h>
#include <SdFat.h>
#include <SFEMP3Shield.h>
#include <Bounce2.h> 

/**
 * The following code defines the 6 analog and 2 digitial pins on the Arduino board used for the 8 adaptive switches. 
 */
#define M_0  A0
#define M_1  A1
#define M_2  A2
#define M_3  A3
#define M_4  A4
#define M_5  A5
#define M_6  10
#define M_7  5
/**
 * \breif Macro for the Debounce Period [milliseconds]
 */
#define BUTTON_DEBOUNCE_PERIOD 20 //ms

/**
 * \brief Object instancing the SdFat library.
 *
 * principal object for handling all SdCard functions.
 */
SdFat sd;

/**
 * \brief Object instancing the SFEMP3Shield library.
 *
 * principal object for handling all the attributes, members and functions for the library.
 */
SFEMP3Shield MP3player;

/**
 * \brief Object instancing the Next Button.
 */
Bounce m_0  = Bounce();
Bounce m_1  = Bounce();
Bounce m_2  = Bounce();
Bounce m_3  = Bounce();
Bounce m_4  = Bounce();
Bounce m_5  = Bounce();
Bounce m_6  = Bounce();
Bounce m_7  = Bounce();
/**
 * \brief Index of the current track playing.
 *
 * Value indicates current playing track, used to populate "x" for playing the 
 * filename of "track00x.mp3" for track000.mp3 through track254.mp3
 */
int8_t current_track = 0;

//------------------------------------------------------------------------------
/**
 * \brief Setup the Arduino Chip's feature for our use.
 *
 * After Arduino's kernel has booted initialize basic features for this
 * application, such as Serial port and MP3player objects with .begin.
 */
void setup() {
  Serial.begin(115200);

  pinMode(M_0, INPUT_PULLUP);
  pinMode(M_1, INPUT_PULLUP);
  pinMode(M_2, INPUT_PULLUP);
  pinMode(M_3, INPUT_PULLUP);
  pinMode(M_4, INPUT_PULLUP);
  pinMode(M_5, INPUT_PULLUP);
  pinMode(M_6, INPUT_PULLUP);
  pinMode(M_7, INPUT_PULLUP);

  m_0.attach(M_0);
  m_0.interval(BUTTON_DEBOUNCE_PERIOD);
  m_1.attach(M_1);
  m_1.interval(BUTTON_DEBOUNCE_PERIOD);
  m_2.attach(M_2);
  m_2.interval(BUTTON_DEBOUNCE_PERIOD);
  m_3.attach(M_3);
  m_3.interval(BUTTON_DEBOUNCE_PERIOD);
  m_4.attach(M_4);
  m_4.interval(BUTTON_DEBOUNCE_PERIOD);
  m_5.attach(M_5);
  m_5.interval(BUTTON_DEBOUNCE_PERIOD);
  m_6.attach(M_6);
  m_6.interval(BUTTON_DEBOUNCE_PERIOD);
  m_7.attach(M_7);
  m_7.interval(BUTTON_DEBOUNCE_PERIOD);

  if(!sd.begin(9, SPI_HALF_SPEED)) sd.initErrorHalt();
  if (!sd.chdir("/")) sd.errorHalt("sd.chdir");

  MP3player.begin();
  MP3player.setVolume(10,10);
  
  Serial.println(F("Looking for Buttons to be depressed..."));
}


//------------------------------------------------------------------------------
/**
 * \brief Main Loop the Arduino Chip
 *
 * This is called at the end of Arduino kernel's main loop before recycling.
 * And is where the user's is executed.
 *
 * \note If the means of refilling is not interrupt based then the
 * MP3player object is serviced with the availaible function.
 */
void loop() {

// Below is only needed if not interrupt driven. Safe to remove if not using.
#if defined(USE_MP3_REFILL_MEANS) \
    && ( (USE_MP3_REFILL_MEANS == USE_MP3_SimpleTimer) \
    ||   (USE_MP3_REFILL_MEANS == USE_MP3_Polled)      )

  MP3player.available();
#endif

//Serial.println("No Button");
  if (m_0.update()) {
    if (m_0.read() == LOW)	{
      Serial.print(F("B_PLAY pressed, Start Playing Track # 1"));
       MP3player.playTrack(1);
       
    }
  }

  if (m_1.update()) {
    if (m_1.read() == LOW)	{
      Serial.print(F("B_STOP pressed, Stopping Track # 2"));
       MP3player.playTrack(2);
    }
  }

  if (m_2.update()) {
    if (m_2.read() == LOW)	{
      Serial.print(F("B_NEXT pressed, Start Playing Next Track # 3"));
      MP3player.playTrack(3);
    }
  }
if (m_3.update()) {
    if (m_3.read() == LOW)  {
      Serial.print(F("B_NEXT pressed, Start Playing Next Track # 4"));
      MP3player.playTrack(4);
    }
  }
  if (m_4.update()) {
    if (m_4.read() == LOW)  {
      Serial.print(F("B_NEXT pressed, Start Playing Next Track # 5"));
      MP3player.playTrack(5);
    }
  }
  if (m_5.update()) {
    if (m_5.read() == LOW)  {
      Serial.print(F("B_NEXT pressed, Start Playing Next Track # 6"));
      MP3player.playTrack(6);
    }
  }
  if (m_6.update()) {
    if (m_6.read() == LOW)  {
      Serial.print(F("B_NEXT pressed, Start Playing Next Track # 7"));
      MP3player.playTrack(7);
    }
  }
  
  if (m_7.update()) {
    if (m_7.read() == LOW)  {
      Serial.print(F("B_NEXT pressed, Start Playing Next Track # 8"));
      MP3player.playTrack(8);
    }
  }

}
